package com.yingxin.beijingvehicleflow.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

/**
 * @auther 李晓阳
 * @email lixy@yingxininfo.net
 * @date 2020/2/26 17:12
 */
public class QRCodeHelper {

    private static final Logger logger = LoggerFactory.getLogger(QRCodeHelper.class);
    private static final int multi = 4;

    //白边为4个模块，是标准，下面做图片放大时候会自动放大
    private static final int WIDTH_BLOCK = 4;

    /**
     * 二维码图流转化为图片对象
     * @param imageData
     * @param width
     * @return
     * @throws IOException
     */
    public static String imgStreamToBase64Image(String imageData, int width) throws IOException {
        //白色背景 whiteBlock*2 上下两个
        int canvasWidth = (width + WIDTH_BLOCK * 2) * multi;
        BufferedImage image = new BufferedImage(canvasWidth, canvasWidth, BufferedImage.TYPE_INT_ARGB);
        image.createGraphics();
        Graphics2D graphics = (Graphics2D) image.getGraphics();
        graphics.fillRect(0, 0, canvasWidth, canvasWidth);
        //base64解码
        byte[] sourceBytes = new BASE64Decoder().decodeBuffer(imageData);
        //填充数据
        int n = width;
        int wid = (n + 7) / 8;
        for (int i = 0; i < n; i++) {
            int ln = i * wid;
            for (int j = 0; j < n; j++) {
                int k = ln + j / 8;
                Color color = (((byte) 0) == (sourceBytes[k] & ((byte) 0x01 << (7 - (j % 8))))) ? Color.WHITE : Color.BLACK;
                graphics.setColor(color);
                graphics.fillRect((i + WIDTH_BLOCK) * multi, (j + WIDTH_BLOCK) * multi, multi, multi);
            }
        }
        ByteArrayOutputStream baos = new ByteArrayOutputStream();//io流
        ImageIO.write(image, "png", baos);//写入流中
        byte[] bytes = baos.toByteArray();//转换成字节
        BASE64Encoder encoder = new BASE64Encoder();
        String png_base64 = encoder.encodeBuffer(bytes).trim();//转换成base64串
        png_base64 = png_base64.replaceAll("\n", "").replaceAll("\r", "");//删除 \r\n
        try {
            if (baos != null) {
                baos.close();
            }
        } catch (IOException e) {
            logger.error("二维码流关闭异常:", e);
        }
        return png_base64;
    }



}
